/*
 * $RCSfile: AnalogOutputSubsystem.hpp,v $
 * $Revision: 1.5 $
 * $Date: 2009/11/29 17:21:05 $
 * jEdit:tabSize=4:collapseFolds=1:
 *
 * class AnalogOutputSubsystem declarations
 */


#if ! defined( AnalogOutputSubsystem_hpp )
#define AnalogOutputSubsystem_hpp

// {{{ includes
#include <DeviceSubsystem.hpp>
#include <OutputVoltagePoint.hpp>
// }}}

namespace AIOUSB {

// {{{ class AnalogOutputSubsystem declarations

/*
 * class AnalogOutputSubsystem represents the analog output subsystem of a board
 */

class AnalogOutputSubsystem : public DeviceSubsystem {
protected:
	static const int MAX_COUNTS;
	static const int NUM_RANGES;
	int range;									// current D/A range
	int numChannels;							// number of D/A channels
	static const double daRange[][ 2 ];

public:
	AnalogOutputSubsystem( USBDevice &parent );
	virtual ~AnalogOutputSubsystem();

	/*
	 * properties
	 */

	virtual std::ostream &print( std::ostream &out );

	int getNumChannels() const {
		return numChannels;
	}	// getNumChannels()

	/*
	 * configuration
	 */

	int getRange() const {
		return range;
	}	// getRange()

	AnalogOutputSubsystem &setRange( int range );

	/*
	 * operations
	 */

	AnalogOutputSubsystem &write( int channel, unsigned short counts );
	AnalogOutputSubsystem &write( const UShortArray &points );
	AnalogOutputSubsystem &writeVolts( int channel, double volts );
	AnalogOutputSubsystem &writeVolts( const OutputVoltagePointArray &points );
	double countsToVolts( unsigned short counts ) const;
	DoubleArray countsToVolts( const UShortArray &counts ) const;
	unsigned short voltsToCounts( double volts ) const;
	UShortArray voltsToCounts( const DoubleArray &volts ) const;
};	// class AnalogOutputSubsystem

// }}}

}	// namespace AIOUSB

#endif


/* end of file */
