/*
 * $RCSfile: CounterSubsystem.hpp,v $
 * $Revision: 1.5 $
 * $Date: 2009/11/23 12:31:25 $
 * jEdit:tabSize=4:collapseFolds=1:
 *
 * class CounterSubsystem declarations
 */


#if ! defined( CounterSubsystem_hpp )
#define CounterSubsystem_hpp

// {{{ includes
#include <DeviceSubsystem.hpp>
// }}}

namespace AIOUSB {

// {{{ class Counter declarations

/*
 * class Counter represents a single counter/timer
 */

class CounterSubsystem;

class Counter {
	friend class CounterSubsystem;

protected:
	CounterSubsystem *parent;					// subsystem that this counter belongs to
	int counterIndex;							// counter index (using 0-based counter addressing)

public:
	// 8254 counter/timer modes
	enum {
		  MODE_TERMINAL_COUNT	= 0				// mode 0: interrupt on terminal count
		, MODE_ONE_SHOT			= 1				// mode 1: hardware retriggerable one-shot
		, MODE_RATE_GENERATOR	= 2				// mode 2: rate generator
		, MODE_SQUARE_WAVE		= 3				// mode 3: square wave mode
		, MODE_SW_TRIGGERED		= 4				// mode 4: software triggered mode
		, MODE_HW_TRIGGERED		= 5				// mode 5: hardware triggered strobe (retriggerable)
	};	// enum

	Counter( CounterSubsystem *parent, int counterIndex );
	int getDeviceIndex() const;
	Counter &setMode( int mode );
	Counter &setCount( unsigned short count );
	Counter &setModeAndCount( int mode, unsigned short count );
	unsigned short readCount();
	UShortArray readCountAndStatus();
	unsigned short readCountAndSetModeAndCount( int mode, unsigned short count );
};	// class Counter

// }}}

// {{{ class CounterList declarations

class CounterList : public std::vector<Counter*> {
};	// class CounterList

// }}}

// {{{ class CounterSubsystem declarations

/*
 * class CounterSubsystem represents the counter/timer subsystem of a board
 */

class CounterSubsystem : public DeviceSubsystem {
protected:
	int numCounterBlocks;						// number of counter blocks
	int numCounters;							// number of individual counters
	CounterList counters;						// list of counters

public:
	CounterSubsystem( USBDevice &parent );
	virtual ~CounterSubsystem();

	/*
	 * properties
	 */

	virtual std::ostream &print( std::ostream &out );

	int getNumCounterBlocks() const {
		return numCounterBlocks;
	}	// getNumCounterBlocks()

	int getNumCounters() const {
		return numCounters;
	}	// getNumCounters()

	/*
	 * operations
	 */

	Counter &getCounter( int counter );
	UShortArray readCounts( bool oldData );
	CounterSubsystem &selectGate( int counter );
	double startClock( int counterBlock, double clockHz );

	CounterSubsystem &stopClock( int counterBlock ) {
		startClock( counterBlock, 0 );
		return *this;
	}	// stopClock()

};	// class CounterSubsystem

// }}}

}	// namespace AIOUSB

#endif


/* end of file */
