/*
 * $RCSfile: DIOStreamSubsystem.cpp,v $
 * $Revision: 1.5 $
 * $Date: 2009/11/23 21:37:35 $
 * jEdit:tabSize=4:indentSize=4:collapseFolds=1:
 *
 * class DIOStreamSubsystem implementation
 */


// {{{ includes
#include <assert.h>
#include <aiousb.h>
#include "USBDeviceManager.hpp"
#include "DIOStreamSubsystem.hpp"
// }}}

using namespace std;

namespace AIOUSB {

// {{{ class DIOStreamSubsystem implementation

DIOStreamSubsystem::DIOStreamSubsystem( USBDevice &parent )
		: DeviceSubsystem( parent ) {
	// nothing to do
}	// DIOStreamSubsystem::DIOStreamSubsystem()


DIOStreamSubsystem::~DIOStreamSubsystem() {
	// nothing to do
}	// DIOStreamSubsystem::~DIOStreamSubsystem()

/*
 * properties
 */

ostream &DIOStreamSubsystem::print( ostream &out ) {
	assert( &out != 0 );
	out << "    Digital I/O streaming capability installed" << endl;
	return out;
}	// DIOStreamSubsystem::print()

/*
 * configuration
 */

DIOStreamSubsystem &DIOStreamSubsystem::setStreamingBlockSize( int blockSize ) {
	const int result = AIOUSB_SetStreamingBlockSize( getDeviceIndex(), blockSize );
	if( result != AIOUSB_SUCCESS )
		throw OperationFailedException( result );
	return *this;
}	// DIOStreamSubsystem::setStreamingBlockSize()


double DIOStreamSubsystem::setClock( bool directionRead, double clockHz ) {
	double unusedClockHz = 0;
	int result;
	if( directionRead )
		result = DIO_StreamSetClocks( getDeviceIndex(), &clockHz, &unusedClockHz );
	else
		result = DIO_StreamSetClocks( getDeviceIndex(), &unusedClockHz, &clockHz );
	if( result != AIOUSB_SUCCESS )
		throw OperationFailedException( result );
	return clockHz;
}	// DIOStreamSubsystem::setClock()

/*
 * operations
 */

DIOStreamSubsystem &DIOStreamSubsystem::open( bool directionRead ) {
	const int result = DIO_StreamOpen( getDeviceIndex(), directionRead );
	if( result != AIOUSB_SUCCESS )
		throw OperationFailedException( result );
	return *this;
}	// DIOStreamSubsystem::open()


DIOStreamSubsystem &DIOStreamSubsystem::close() {
	const int result = DIO_StreamClose( getDeviceIndex() );
	if( result != AIOUSB_SUCCESS )
		throw OperationFailedException( result );
	return *this;
}	// DIOStreamSubsystem::close()


UShortArray DIOStreamSubsystem::read( int numSamples ) {
	if( numSamples < 1 )
		throw IllegalArgumentException( "Invalid number of samples" );
	UShortArray values( numSamples );
	unsigned long bytesTransferred;
	const int result = DIO_StreamFrame( getDeviceIndex(), values.size(), values.data(), &bytesTransferred );
	if( result != AIOUSB_SUCCESS )
		throw OperationFailedException( result );
	const int samplesTransferred = bytesTransferred / sizeof( unsigned short );
	if( samplesTransferred != numSamples )
		values.resize( samplesTransferred );
	return values;
}	// DIOStreamSubsystem::read()


int DIOStreamSubsystem::write( const UShortArray &values ) {
	if(
		&values == 0
		|| values.size() < 1
	)
		throw IllegalArgumentException( "Invalid values" );
	unsigned long bytesTransferred;
	const int result = DIO_StreamFrame( getDeviceIndex(), values.size(), ( unsigned short * ) values.data(), &bytesTransferred );
	if( result != AIOUSB_SUCCESS )
		throw OperationFailedException( result );
	return bytesTransferred / sizeof( unsigned short );
}	// DIOStreamSubsystem::write()

// }}}

}	// namespace AIOUSB


/* end of file */
