/*
 * $RCSfile: DigitalIOSubsystem.hpp,v $
 * $Revision: 1.8 $
 * $Date: 2009/11/30 21:33:39 $
 * jEdit:tabSize=4:collapseFolds=1:
 *
 * class DigitalIOSubsystem declarations
 */


#if ! defined( DigitalIOSubsystem_hpp )
#define DigitalIOSubsystem_hpp

// {{{ includes
#include <DeviceSubsystem.hpp>
// }}}

namespace AIOUSB {

// {{{ class DigitalIOSubsystem declarations

/*
 * class DigitalIOSubsystem represents the digital I/O subsystem of a board
 */

class DigitalIOSubsystem : public DeviceSubsystem {
protected:
	int numPorts;								// number of digital I/O ports (bytes)
	int numChannels;							// number of digital I/O channels (bits)
	int numTristateGroups;						// number of tristate groups (bytes)
	int numTristates;							// number of tristate bits
	UCharArray writeValues;						// last values written to device

	UCharArray &bitsToBytes( UCharArray &dest, int bit, const BoolArray &src );
	BoolArray &bytesToBits( BoolArray &dest, const UCharArray &src, int bit );

public:
	DigitalIOSubsystem( USBDevice &parent );
	virtual ~DigitalIOSubsystem();

	/*
	 * properties
	 */

	virtual std::ostream &print( std::ostream &out );

	int getNumPorts() const {
		return numPorts;
	}	// getNumPorts()


	int getNumChannels() const {
		return numChannels;
	}	// getNumChannels()


	int getNumTristateGroups() const {
		return numTristateGroups;
	}	// getNumTristateGroups()


	int getNumTristates() const {
		return numTristates;
	}	// getNumTristates()

	/*
	 * configuration
	 */

	DigitalIOSubsystem &configure( bool tristate, const BoolArray &outputs, const BoolArray &values );
	DigitalIOSubsystem &configure( const BoolArray &tristates, const BoolArray &outputs, const BoolArray &values );
	DigitalIOSubsystem &getConfiguration( BoolArray &tristates, BoolArray &outputs );

	/*
	 * operations
	 */

	bool read( int channel );
	BoolArray read( int startChannel, int numChannels );
	DigitalIOSubsystem &write( int channel, bool value );
	DigitalIOSubsystem &write( int startChannel, const BoolArray &values );
};	// class DigitalIOSubsystem

// }}}

}	// namespace AIOUSB

#endif


/* end of file */
