/*
 * $RCSfile: OutputVoltagePoint.hpp,v $
 * $Revision: 1.2 $
 * $Date: 2009/11/29 17:21:26 $
 * jEdit:tabSize=4:collapseFolds=1:
 *
 * class OutputVoltagePoint, OutputVoltagePointArray declarations
 */


#if ! defined( OutputVoltagePoint_hpp )
#define OutputVoltagePoint_hpp

// {{{ includes
#include <vector>
// }}}

namespace AIOUSB {

// {{{ class OutputVoltagePoint declarations

/**
 * Class OutputVoltagePoint represents a single analog output data point, consisting of a D/A
 * channel number and a voltage to output to that channel. It is used by method
 * <i>{@link AnalogOutputSubsystem#write( const OutputVoltagePointArray &points )}</i> to output a series
 * of voltages to multiple D/A channels.
 */

class OutputVoltagePoint {
public:

	/** Channel number to output voltage to. */
	int channel;

	/** Voltage to output. */
	double volts;

	/**
	 * Default constructor for analog output data point.
	 */

	OutputVoltagePoint() {
		// nothing to do
	}	// OutputVoltagePoint()

	/**
	 * Constructor for analog output data point.
	 * @param channel the channel number to output voltage to.
	 * @param volts the voltage to output.
	 */

	OutputVoltagePoint( int channel, double volts ) {
		this->channel = channel;
		this->volts = volts;
	}	// OutputVoltagePoint()

};	// class OutputVoltagePoint

// }}}

// {{{ class OutputVoltagePointArray declarations

class OutputVoltagePointArray : public std::vector<OutputVoltagePoint> {
public:
	OutputVoltagePointArray( int size = 0 ) : std::vector<OutputVoltagePoint>( size ) {}
};	// class OutputVoltagePointArray

// }}}

}	// namespace AIOUSB

#endif


/* end of file */
