/*
 * $RCSfile: USBDevice.hpp,v $
 * $Revision: 1.9 $
 * $Date: 2009/11/22 19:07:37 $
 * jEdit:tabSize=4:indentSize=4:collapseFolds=1:
 *
 * class BoolArray, UCharArray, UShortArray, IntArray, DoubleArray, USBDevice declarations
 */


#if ! defined( USBDevice_hpp )
#define USBDevice_hpp

// {{{ includes
#include <vector>
#include <string>
#include <iostream>
// }}}

namespace AIOUSB {

// {{{ class BoolArray, UCharArray, UShortArray, IntArray, DoubleArray declarations

class BoolArray : public std::vector<bool> {
public:
	BoolArray( int size = 0 ) : std::vector<bool>( size ) {}
};	// class BoolArray

class UCharArray : public std::vector<unsigned char> {
public:
	UCharArray( int size = 0 ) : std::vector<unsigned char>( size ) {}
};	// class UCharArray

class UShortArray : public std::vector<unsigned short> {
public:
	UShortArray( int size = 0 ) : std::vector<unsigned short>( size ) {}
};	// class UShortArray

class IntArray : public std::vector<int> {
public:
	IntArray( int size = 0 ) : std::vector<int>( size ) {}
};	// class IntArray

class DoubleArray : public std::vector<double> {
public:
	DoubleArray( int size = 0 ) : std::vector<double>( size ) {}
};	// class DoubleArray

// }}}

// {{{ class USBDevice declarations

/*
 * class USBDevice is the base class for all ACCES USB device classes;
 * this class hierarchy is designed to be a fairly lightweight wrapper around
 * the AIOUSB library; the primary goal of this class hierarchy is to provide
 * a pure C++ interface, as well as to facilitate using AIOUSB
 */

class USBDevice {
	friend class DIOStreamSubsystem;
	friend class AnalogInputSubsystem;

public:
	static const int CUSTOM_EEPROM_SIZE;

protected:
	int deviceIndex;							// device index on bus
	int productID;								// 16-bit product ID
	std::string name;							// device name
	__uint64_t serialNumber;					// 64-bit serial number or 0

	USBDevice &clearFIFO( int method );
	USBDevice &setMiscClock( double clockHz );

public:
	USBDevice( int productID, int deviceIndex );
	virtual ~USBDevice();

	/*
	 * properties
	 */

	virtual std::ostream &print( std::ostream &out );

	int getDeviceIndex() const {
		return deviceIndex;
	}	// getDeviceIndex()

	int getProductID() const {
		return productID;
	}	// getProductID()

	const std::string &getName() const {
		return name;
	}	// getName()

	__uint64_t getSerialNumber() const {
		return serialNumber;
	}	// getSerialNumber()

	/*
	 * configuration
	 */

	int getCommTimeout() const;
	USBDevice &setCommTimeout( int timeout );

	/*
	 * operations
	 */

	USBDevice &reset();
	USBDevice &customEEPROMWrite( int address, const UCharArray &data );
	UCharArray customEEPROMRead( int address, int numBytes );
};	// class USBDevice

// }}}

// {{{ function declarations

extern std::ostream &operator<<( std::ostream &out, USBDevice &device );
extern std::ostream &operator<<( std::ostream &out, USBDevice *device );

// }}}

}	// namespace AIOUSB

#endif


/* end of file */
