/*
 * $RCSfile: USBDeviceManager.hpp,v $
 * $Revision: 1.6 $
 * $Date: 2009/11/26 22:06:44 $
 * jEdit:tabSize=4:indentSize=4:collapseFolds=1:
 *
 * class DeviceList, USBDeviceManager, OperationFailedException, IllegalArgumentException declarations
 */


#if ! defined( USBDeviceManager_hpp )
#define USBDeviceManager_hpp

// {{{ includes
#include <iostream>
#include <stdexcept>
#include <string>
#include <vector>
#include <aiousb.h>
// }}}

namespace AIOUSB {

// {{{ class DeviceList declarations

class USBDevice;

class DeviceList : public std::vector<USBDevice*> {
};	// class DeviceList

// }}}

// {{{ class USBDeviceManager declarations

class USBDeviceManager {
public:
	const std::string VERSION_NUMBER;
	const std::string VERSION_DATE;

protected:
	const std::string MESSAGE_NOT_OPEN;
	const unsigned long OPEN_PATTERN;			// unique, random value
	unsigned long openStatus;					// if openStatus == OPEN_PATTERN, module is initialized
	DeviceList deviceList;						// contains all the ACCES devices found on the bus

	void emptyDeviceList();

public:
	USBDeviceManager();
	virtual ~USBDeviceManager();

	/*
	 * properties
	 */

	virtual std::ostream &print( std::ostream &out );
	USBDeviceManager &printDevices();

	std::string getAIOUSBVersion() const {
		return AIOUSB_GetVersion();
	}	// getAIOUSBVersion()

	std::string getAIOUSBVersionDate() const {
		return AIOUSB_GetVersionDate();
	}	// getAIOUSBVersionDate()

	/*
	 * operations
	 */

	void listDevices() const {
		AIOUSB_ListDevices();
	}	// listDevices()

	static std::string getResultCodeAsString( int result ) {
		return AIOUSB_GetResultCodeAsString( result );
	}	// getResultCodeAsString()

	bool isOpen() const {
		return openStatus == OPEN_PATTERN;
	}	// isOpen()

	USBDeviceManager &open();
	USBDeviceManager &close();
	USBDeviceManager &scanForDevices();
	DeviceList getDeviceByProductID( int productID ) const;
	DeviceList getDeviceByProductID( int minProductID, int maxProductID ) const;
	DeviceList getDeviceBySerialNumber( __uint64_t serialNumber ) const;
};	// class USBDeviceManager

// }}}

// {{{ class OperationFailedException declarations

class OperationFailedException : public std::runtime_error {
public:
	OperationFailedException( const std::string &message ) : std::runtime_error( message ) {}
	OperationFailedException( int result ) : std::runtime_error( USBDeviceManager::getResultCodeAsString( result ) ) {}
};	// class OperationFailedException

// }}}

// {{{ class IllegalArgumentException declarations

class IllegalArgumentException : public std::invalid_argument {
public:
	IllegalArgumentException( const std::string &message ) : std::invalid_argument( message ) {}
};	// class IllegalArgumentException

// }}}

}	// namespace AIOUSB

#endif


/* end of file */
