/*
 * $RCSfile: USBDeviceManager.java,v $
 * $Date: 2009/11/30 20:06:50 $
 * $Revision: 1.14 $
 * jEdit:tabSize=4:indentSize=4:collapseFolds=1:
 */

package com.acces.aiousb;

// {{{ imports
import java.io.*;
import java.lang.*;
import java.util.*;
// }}}

/**
 * Class USBDeviceManager manages all the USB devices on the bus. It scans the bus and builds
 * a list of all the devices found. It also initializes and terminates use of the underlying
 * AIOUSB module.
 */

public class USBDeviceManager {

	// {{{ public constants
	/** The version number of this Java class library. */
	public static final String VERSION_NUMBER = "1.3";

	/** The version date of this Java class library. */
	public static final String VERSION_DATE = "30 November 2009";

	/** Indicates that underlying AIOUSB module operation succeeded. */
	public static final int SUCCESS = 0;
	// }}}

	// {{{ product IDs
	/*
	 * these product IDs are constant
	 */
	public static final int USB_DA12_8A_REV_A			= 0x4001;
	public static final int USB_DA12_8A					= 0x4002;
	public static final int USB_DA12_8E					= 0x4003;

	/*
	 * these are the product IDs after firmware is uploaded to the device; prior
	 * to uploading the firmware, the product ID is that shown below minus 0x8000
	 */
	public static final int USB_DIO_32					= 0x8001;
	public static final int USB_DIO_48					= 0x8002;
	public static final int USB_DIO_96					= 0x8003;
	public static final int USB_DI16A_REV_A1			= 0x8008;
	public static final int USB_DO16A_REV_A1			= 0x8009;
	public static final int USB_DI16A_REV_A2			= 0x800a;
	public static final int USB_DIO_16H					= 0x800c;
	public static final int USB_DI16A					= 0x800d;
	public static final int USB_DO16A					= 0x800e;
	public static final int USB_DIO_16A					= 0x800f;
	public static final int USB_IIRO_16					= 0x8010;
	public static final int USB_II_16					= 0x8011;
	public static final int USB_RO_16					= 0x8012;
	public static final int USB_IIRO_8					= 0x8014;
	public static final int USB_II_8					= 0x8015;
	public static final int USB_IIRO_4					= 0x8016;
	public static final int USB_IDIO_16					= 0x8018;
	public static final int USB_II_16_OLD				= 0x8019;
	public static final int USB_IDO_16					= 0x801a;
	public static final int USB_IDIO_8					= 0x801c;
	public static final int USB_II_8_OLD				= 0x801d;
	public static final int USB_IDIO_4					= 0x801e;
	public static final int USB_CTR_15					= 0x8020;
	public static final int USB_IIRO4_2SM				= 0x8030;
	public static final int USB_IIRO4_COM				= 0x8031;
	public static final int USB_DIO16RO8				= 0x8032;
	public static final int PICO_DIO16RO8				= 0x8033;
	public static final int USB_AI16_16A				= 0x8040;
	public static final int USB_AI16_16E				= 0x8041;
	public static final int USB_AI12_16A				= 0x8042;
	public static final int USB_AI12_16					= 0x8043;
	public static final int USB_AI12_16E				= 0x8044;
	public static final int USB_AI16_64MA				= 0x8045;
	public static final int USB_AI16_64ME				= 0x8046;
	public static final int USB_AI12_64MA				= 0x8047;
	public static final int USB_AI12_64M				= 0x8048;
	public static final int USB_AI12_64ME				= 0x8049;
	public static final int USB_AI16_32A				= 0x804a;
	public static final int USB_AI16_32E				= 0x804b;
	public static final int USB_AI12_32A				= 0x804c;
	public static final int USB_AI12_32					= 0x804d;
	public static final int USB_AI12_32E				= 0x804e;
	public static final int USB_AI16_64A				= 0x804f;
	public static final int USB_AI16_64E				= 0x8050;
	public static final int USB_AI12_64A				= 0x8051;
	public static final int USB_AI12_64					= 0x8052;
	public static final int USB_AI12_64E				= 0x8053;
	public static final int USB_AI16_96A				= 0x8054;
	public static final int USB_AI16_96E				= 0x8055;
	public static final int USB_AI12_96A				= 0x8056;
	public static final int USB_AI12_96					= 0x8057;
	public static final int USB_AI12_96E				= 0x8058;
	public static final int USB_AI16_128A				= 0x8059;
	public static final int USB_AI16_128E				= 0x805a;
	public static final int USB_AI12_128A				= 0x805b;
	public static final int USB_AI12_128				= 0x805c;
	public static final int USB_AI12_128E				= 0x805d;
	public static final int USB_AO16_16A				= 0x8060;
	public static final int USB_AO16_16					= 0x8061;
	public static final int USB_AO16_12A				= 0x8062;
	public static final int USB_AO16_12					= 0x8063;
	public static final int USB_AO16_8A					= 0x8064;
	public static final int USB_AO16_8					= 0x8065;
	public static final int USB_AO16_4A					= 0x8066;
	public static final int USB_AO16_4					= 0x8067;
	public static final int USB_AO12_16A				= 0x8068;
	public static final int USB_AO12_16					= 0x8069;
	public static final int USB_AO12_12A				= 0x806a;
	public static final int USB_AO12_12					= 0x806b;
	public static final int USB_AO12_8A					= 0x806c;
	public static final int USB_AO12_8					= 0x806d;
	public static final int USB_AO12_4A					= 0x806e;
	public static final int USB_AO12_4					= 0x806f;
	// }}}

	// {{{ protected members
	protected Vector<USBDevice> deviceList = new Vector<USBDevice>();
	protected static final int OPEN_PATTERN = 0x786938f5;	// unique, random value
	protected int openStatus = 0;				// if openStatus == OPEN_PATTERN, module is initialized
	protected static final String MESSAGE_NOT_OPEN = "Not open, must call open() first";
	// }}}

	// {{{ static initializers
	static {
		System.loadLibrary( "javaaiousb" );		// libjavaaiousb.so
	}	// static
	// }}}

	// {{{ protected methods

	protected native int init();				// => unsigned long AIOUSB_Init()
	protected native void exit();				// => void AIOUSB_Exit()
	protected native int getDevices();			// => unsigned long GetDevices( void )
	protected native int getProductID( int deviceIndex );
	protected native int getDeviceByProductID( int minProductID, int maxProductID, int[] devices );
		// => unsigned long GetDeviceByProductID( int minProductID, int maxProductID, int maxDevices, int *deviceList )

	protected void finalize() throws Throwable {
		if( isOpen() )
			close();							// in case user forgot
	}	// finalize()

	// }}}

	// {{{ public methods

	/**
	 * Constructor for USB device manager. You must call <i>{@link #open() open()}</i> before using the device manager.
	 */

	public USBDeviceManager() {
		// nothing to do
	}	// USBDeviceManager()

	/*
	 * properties
	 */

	/**
	 * Prints the properties of this device manager and all of the devices found on the bus to
	 * the specified print stream. Mainly useful for diagnostic purposes.
	 * @param stream the print stream where properties will be printed.
	 * @return The print stream.
	 */

	public PrintStream print( PrintStream stream ) {
		if( ! isOpen() )
			throw new OperationFailedException( MESSAGE_NOT_OPEN );
		if( deviceList.size() > 0 ) {
			stream.println( "ACCES devices found:" );
			for( USBDevice device : deviceList )
				device.print( stream );
		} else
			stream.println( "No ACCES devices found" );
		return stream;
	}	// print()

	/**
	 * Prints the properties of this device manager and all of the devices found on the bus to
	 * the standard output device. Mainly useful for diagnostic purposes.
	 * @return This device manager, useful for chaining together multiple operations.
	 * @throws OperationFailedException
	 */

	public USBDeviceManager printDevices() {
		print( System.out );
		return this;
	}	// printDevices()

	/**
	 * Gets the version number of the underlying AIOUSB module.
	 * @return The AIOUSB module version number as a string with the form, "1.78".
	 */

	public native String getAIOUSBVersion();		// => const char *AIOUSB_GetVersion()

	/**
	 * Gets the version date of the underlying AIOUSB module.
	 * @return The AIOUSB module version date as a string with the form, "15 November 2009".
	 */

	public native String getAIOUSBVersionDate();	// => const char *AIOUSB_GetVersionDate()

	/*
	 * operations
	 */

	/**
	 * Prints the properties of all the devices found on the bus to the standard output device.
	 * This function is similar to <i>{@link #printDevices() printDevices()}</i> but is implemented
	 * by the underlying AIOUSB module and produces different output than <i>printDevices()</i>. Mainly
	 * useful for diagnostic purposes.
	 */

	public native void listDevices();			// => void AIOUSB_ListDevices()

	/**
	 * Gets the string representation of an AIOUSB result code, useful mainly for debugging purposes.
	 * This method is also used to convert an AIOUSB result code to a string when a
	 * {@link OperationFailedException} is thrown in response to an AIOUSB failure.
	 * @param result an AIOUSB result code.
	 * @return The string representation of <i>result</i>.
	 */

	public static native String getResultCodeAsString( int result );
		// => const char *AIOUSB_GetResultCodeAsString( unsigned long result )

	/**
	 * Tells if the USB device manager has been "opened" for use <i>(see {@link #open() open()})</i>.
	 * @return <i>True</i> indicates that the device manager is open and ready to be used; <i>false</i>
	 * indicates that it is not open.
	 */

	public boolean isOpen() {
		return openStatus == OPEN_PATTERN;
	}	// isOpen()

	/**
	 * "Opens" the USB device manager for use. Before the USB device manager may be used, <i>open()</i> must be
	 * called. <i>Open()</i> initializes the underlying AIOUSB module and scans the bus for devices, building
	 * a list of the devices found. When finished using the USB device manager, <i>{@link #close() close()}</i>
	 * must be called. It is possible to call <i>close()</i> and then call <i>open()</i> again, which effectively
	 * reinitializes everything.
	 * @return This device manager, useful for chaining together multiple operations.
	 * @throws OperationFailedException
	 */

	public USBDeviceManager open() {
		if( ! isOpen() ) {
			final int result = init();
			if( result == SUCCESS ) {
				openStatus = OPEN_PATTERN;
				scanForDevices();
			} else
				throw new OperationFailedException( "Initialization failed: " + getResultCodeAsString( result ) );
		} else
			throw new OperationFailedException( "Already open" );
		return this;
	}	// open()

	/**
	 * "Closes" the USB device manager for use. When finished using the USB device manager, and assuming
	 * <i>{@link #open() open()}</i> was properly called, <i>close()</i> must be called. <i>Close()</i> terminates
	 * use of the underlying AIOUSB module and discards the list of devices found. <i>You must terminate use of
	 * all USB devices before calling close()!</i> You can call <i>open()</i> again to reinitialize things
	 * and reestablish connections to USB devices.
	 * @return This device manager, useful for chaining together multiple operations.
	 * @throws OperationFailedException
	 */

	public USBDeviceManager close() {
		if( isOpen() ) {
			exit();
			openStatus = 0;
			deviceList = new Vector<USBDevice>();
		} else
			throw new OperationFailedException( MESSAGE_NOT_OPEN );
		return this;
	}	// close()

	/**
	 * Re-scans the bus for devices. <i>ScanForDevices()</i> is called automatically by
	 * <i>{@link #open() open()}</i>. <i>You must terminate use of all USB devices before calling
	 * scanForDevices()!</i> After calling <i>scanForDevices()</i> you can reestablish
	 * connections to USB devices.
	 * @return This device manager, useful for chaining together multiple operations.
	 * @throws OperationFailedException
	 */

	public USBDeviceManager scanForDevices() {
		if( isOpen() ) {
			deviceList.clear();
			final int MAX_DEVICES = 100;
			int devices[] = new int[ 1 + MAX_DEVICES * 2 ];		// device index-product ID pairs
			final int result = getDeviceByProductID( 0, 0xffff, devices );	// get all devices
			if( result != SUCCESS )
				throw new OperationFailedException( result );
			final int numDevices = devices[ 0 ];
			for( int index = 0; index < numDevices; index++ ) {
				USBDevice device = null;
				final int deviceIndex = devices[ 1 + index * 2 ];
				final int productID = devices[ 1 + index * 2 + 1 ];
				if( productID == USB_CTR_15 ) {
					device = new USB_CTR_15_Family( productID, deviceIndex );
				} else if( productID == USB_DIO_32 ) {
					device = new USB_DIO_32_Family( productID, deviceIndex );
				} else if(
					productID == USB_DIO_48
					|| productID == USB_DIO_96
					|| productID == USB_IIRO_16
					|| productID == USB_II_16
					|| productID == USB_RO_16
					|| productID == USB_IIRO_8
					|| productID == USB_II_8
					|| productID == USB_IIRO_4
					|| productID == USB_IDIO_16
					|| productID == USB_II_16_OLD
					|| productID == USB_IDO_16
					|| productID == USB_IDIO_8
					|| productID == USB_II_8_OLD
					|| productID == USB_IDIO_4
					|| productID == USB_IIRO4_2SM
					|| productID == USB_IIRO4_COM
					|| productID == USB_DIO16RO8
					|| productID == PICO_DIO16RO8
				) {
					device = new USB_DIO_Family( productID, deviceIndex );
				} else if(
					(
						productID >= USB_DI16A_REV_A1
						&& productID <= USB_DI16A_REV_A2
					)
					|| (
						productID >= USB_DIO_16H
						&& productID <= USB_DIO_16A
					)
				) {
					device = new USB_DIO_16_Family( productID, deviceIndex );
				} else if(
					productID >= USB_AI16_16A
					&& productID <= USB_AI12_128E
				) {
					device = new USB_AI16_Family( productID, deviceIndex );
				} else if(
					productID >= USB_AO16_16A
					&& productID <= USB_AO12_4
				) {
					device = new USB_AO16_Family( productID, deviceIndex );
				}	// if( productID ...
				if( device != null )
					deviceList.add( device );
			}	// for( int index ...
		} else
			throw new OperationFailedException( MESSAGE_NOT_OPEN );
		return this;
	}	// scanForDevices()

	/**
	 * Gets a list of all the devices found on the bus matching the specified product ID. Only devices exactly
	 * matching the specified product ID will be returned.
	 * @param productID the product ID to search for.
	 * @return A vector of all the devices found. If no devices were found matching the specified
	 * product ID, the vector will be empty (i.e. contain zero items).
	 */

	public Vector<USBDevice> getDeviceByProductID( int productID ) {
		return getDeviceByProductID( productID, productID );
	}	// getDeviceByProductID()

	/**
	 * Gets a list of all the devices found on the bus matching the specified product ID range. Any device
	 * with a product ID greater than or equal to <i>minProductID</i> and less than or equal to <i>maxProductID</i>
	 * will be returned. You can obtain the entire list of devices detected by passing a value of 0 for
	 * <i>minProductID</i> and a value of 0xffff for <i>maxProductID</i>. Then you can search the list obtained
	 * using your own search criteria.
	 * @param minProductID the minimum product ID to search for.
	 * @param maxProductID the maximum product ID to search for.
	 * @return A vector of all the devices found. If no devices were found matching the specified
	 * product ID range, the vector will be empty (i.e. contain zero items).
	 * @throws IllegalArgumentException
	 */

	public Vector<USBDevice> getDeviceByProductID( int minProductID, int maxProductID ) {
		if(
			minProductID < 0
			|| minProductID > 0xffff
			|| maxProductID < minProductID
			|| maxProductID > 0xffff
		)
			throw new IllegalArgumentException( "Invalid product IDs: " + minProductID + ", " + maxProductID );
		Vector<USBDevice> devices = new Vector<USBDevice>();
		for( int index = 0; index < deviceList.size(); index++ ) {
			final int productID = deviceList.get( index ).getProductID();
			if(
				productID >= minProductID
				&& productID <= maxProductID
			)
				devices.add( deviceList.get( index ) );
		}	// for( int index ...
		return devices;
	}	// getDeviceByProductID()

	/**
	 * Gets a list of all the devices found on the bus matching the specified serial number. Only devices exactly
	 * matching the specified serial number will be returned. In theory, there ought to be only one device
	 * matching a given serial number, but this method returns a vector in order to be consistent with the other
	 * search methods, and in unlikely event that multiple devices do share the same serial number.
	 * @param serialNumber the serial number to search for.
	 * @return A vector of all the devices found. If no devices were found matching the specified
	 * serial number, the vector will be empty (i.e. contain zero items).
	 */

	public Vector<USBDevice> getDeviceBySerialNumber( long serialNumber ) {
		Vector<USBDevice> devices = new Vector<USBDevice>();
		for( int index = 0; index < deviceList.size(); index++ ) {
			if( deviceList.get( index ).getSerialNumber() == serialNumber )
				devices.add( deviceList.get( index ) );
		}	// for( int index ...
		return devices;
	}	// getDeviceBySerialNumber()

	// }}}

}	// class USBDeviceManager

/* end of file */
