/*
 * $RCSfile: USB_DIO_16_Family.java,v $
 * $Date: 2009/11/30 17:52:11 $
 * $Revision: 1.2 $
 * jEdit:tabSize=4:indentSize=4:collapseFolds=1:
 */

package com.acces.aiousb;

// {{{ imports
import java.io.*;
// }}}

/**
 * Class USB_DIO_16_Family represents a USB-DIO-16-family device, which encompasses the following product IDs:
 * <ul>
 * <li>{@link USBDeviceManager#USB_DI16A_REV_A1 USB_DI16A_REV_A1} to {@link USBDeviceManager#USB_DI16A_REV_A2 USB_DI16A_REV_A2}
 * <li>{@link USBDeviceManager#USB_DIO_16H USB_DIO_16H} to {@link USBDeviceManager#USB_DIO_16A USB_DIO_16A}
 * </ul>
 */

public class USB_DIO_16_Family extends USBDevice {

	// {{{ protected members
	protected DigitalIOSubsystem digitalIOSubsystem;
	protected DIOStreamSubsystem dioStreamSubsystem;
	// }}}

	// {{{ public methods

	/**
	 * Constructor for USB device. This constructor should not be called explicitly. Instances
	 * of class <i>USB_DIO_16_Family</i> are automatically created by the USB device manager when
	 * they are detected on the bus. You should use one of the
	 * <i>{@link USBDeviceManager#getDeviceByProductID( int productID ) USBDeviceManager.getDeviceByProductID()}</i>
	 * search methods to obtain a reference to a <i>USB_DIO_16_Family</i> instance. You can then cast the <i>{@link USBDevice}</i>
	 * reference obtained from one of those methods to a <i>USB_DIO_16_Family</i> and make use of this class' methods, like so:<br><br>
	 * <i>Vector&lt;USBDevice&gt; devices = deviceManager.getDeviceByProductID( deviceManager.USB_AI16_16A, deviceManager.USB_AI12_128E );<br>
	 * if( devices.size() > 0 )<br>
	 * &nbsp;&nbsp;USB_DIO_16_Family device = ( USB_DIO_16_Family ) devices.get( 0 );</i>
	 * @param productID the product ID of the device.
	 * @param deviceIndex the index of the device on the USB bus.
	 * @throws IllegalArgumentException
	 */

	public USB_DIO_16_Family( int productID, int deviceIndex ) {
		super( productID, deviceIndex );
		if(
			! (
				(
					productID >= USBDeviceManager.USB_DI16A_REV_A1
					&& productID <= USBDeviceManager.USB_DI16A_REV_A2
				)
				|| (
					productID >= USBDeviceManager.USB_DIO_16H
					&& productID <= USBDeviceManager.USB_DIO_16A
				)
			)
		)
			throw new IllegalArgumentException( "Invalid product ID: " + productID );
		digitalIOSubsystem = new DigitalIOSubsystem( this );
		dioStreamSubsystem = new DIOStreamSubsystem( this );
	}	// USB_DIO_16_Family()

	/**
	 * Gets a reference to the digital I/O subsystem of this device.
	 * @return reference to the digital I/O subsystem.
	 */

	public DigitalIOSubsystem dio() {
		return digitalIOSubsystem;
	}	// dio()

	/**
	 * Gets a reference to the digital I/O streaming subsystem of this device.
	 * @return reference to the digital I/O streaming subsystem.
	 */

	public DIOStreamSubsystem diostream() {
		return dioStreamSubsystem;
	}	// diostream()

	/**
	 * Prints the properties of this device and all of its subsystems. Mainly useful for diagnostic purposes.
	 * @param stream the print stream where properties will be printed.
	 * @return The print stream.
	 */

	public PrintStream print( PrintStream stream ) {
		super.print( stream );
		digitalIOSubsystem.print( stream );
		dioStreamSubsystem.print( stream );
		return stream;
	}	// print()

	// }}}

}	// class USB_DIO_16_Family

/* end of file */
