/*
 * $RCSfile: USB_DIO_Family.java,v $
 * $Date: 2009/11/30 17:52:11 $
 * $Revision: 1.2 $
 * jEdit:tabSize=4:indentSize=4:collapseFolds=1:
 */

package com.acces.aiousb;

// {{{ imports
import java.io.*;
// }}}

/**
 * Class USB_DIO_Family represents a USB-DIO-family device, which performs basic digital I/O and
 * encompasses the following product IDs:
 * <ul>
 * <li>{@link USBDeviceManager#USB_DIO_48 USB_DIO_48}
 * <li>{@link USBDeviceManager#USB_DIO_96 USB_DIO_96}
 * <li>{@link USBDeviceManager#USB_IIRO_16 USB_IIRO_16}
 * <li>{@link USBDeviceManager#USB_II_16 USB_II_16}
 * <li>{@link USBDeviceManager#USB_RO_16 USB_RO_16}
 * <li>{@link USBDeviceManager#USB_IIRO_8 USB_IIRO_8}
 * <li>{@link USBDeviceManager#USB_II_8 USB_II_8}
 * <li>{@link USBDeviceManager#USB_IIRO_4 USB_IIRO_4}
 * <li>{@link USBDeviceManager#USB_IDIO_16 USB_IDIO_16}
 * <li>{@link USBDeviceManager#USB_II_16_OLD USB_II_16_OLD}
 * <li>{@link USBDeviceManager#USB_IDO_16 USB_IDO_16}
 * <li>{@link USBDeviceManager#USB_IDIO_8 USB_IDIO_8}
 * <li>{@link USBDeviceManager#USB_II_8_OLD USB_II_8_OLD}
 * <li>{@link USBDeviceManager#USB_IDIO_4 USB_IDIO_4}
 * <li>{@link USBDeviceManager#USB_IIRO4_2SM USB_IIRO4_2SM}
 * <li>{@link USBDeviceManager#USB_IIRO4_COM USB_IIRO4_COM}
 * <li>{@link USBDeviceManager#USB_DIO16RO8 USB_DIO16RO8}
 * <li>{@link USBDeviceManager#PICO_DIO16RO8 PICO_DIO16RO8}
 * </ul>
 */

public class USB_DIO_Family extends USBDevice {

	// {{{ protected members
	protected DigitalIOSubsystem digitalIOSubsystem;
	// }}}

	// {{{ public methods

	/**
	 * Constructor for USB device. This constructor should not be called explicitly. Instances
	 * of class <i>USB_DIO_Family</i> are automatically created by the USB device manager when
	 * they are detected on the bus. You should use one of the
	 * <i>{@link USBDeviceManager#getDeviceByProductID( int productID ) USBDeviceManager.getDeviceByProductID()}</i>
	 * search methods to obtain a reference to a <i>USB_DIO_Family</i> instance. You can then cast the <i>{@link USBDevice}</i>
	 * reference obtained from one of those methods to a <i>USB_DIO_Family</i> and make use of this class' methods, like so:<br><br>
	 * <i>Vector&lt;USBDevice&gt; devices = deviceManager.getDeviceByProductID( deviceManager.USB_AI16_16A, deviceManager.USB_AI12_128E );<br>
	 * if( devices.size() > 0 )<br>
	 * &nbsp;&nbsp;USB_DIO_Family device = ( USB_DIO_Family ) devices.get( 0 );</i>
	 * @param productID the product ID of the device.
	 * @param deviceIndex the index of the device on the USB bus.
	 * @throws IllegalArgumentException
	 */

	public USB_DIO_Family( int productID, int deviceIndex ) {
		super( productID, deviceIndex );
		if(
			! (
				productID == USBDeviceManager.USB_DIO_48
				|| productID == USBDeviceManager.USB_DIO_96
				|| productID == USBDeviceManager.USB_IIRO_16
				|| productID == USBDeviceManager.USB_II_16
				|| productID == USBDeviceManager.USB_RO_16
				|| productID == USBDeviceManager.USB_IIRO_8
				|| productID == USBDeviceManager.USB_II_8
				|| productID == USBDeviceManager.USB_IIRO_4
				|| productID == USBDeviceManager.USB_IDIO_16
				|| productID == USBDeviceManager.USB_II_16_OLD
				|| productID == USBDeviceManager.USB_IDO_16
				|| productID == USBDeviceManager.USB_IDIO_8
				|| productID == USBDeviceManager.USB_II_8_OLD
				|| productID == USBDeviceManager.USB_IDIO_4
				|| productID == USBDeviceManager.USB_IIRO4_2SM
				|| productID == USBDeviceManager.USB_IIRO4_COM
				|| productID == USBDeviceManager.USB_DIO16RO8
				|| productID == USBDeviceManager.PICO_DIO16RO8
			)
		)
			throw new IllegalArgumentException( "Invalid product ID: " + productID );
		digitalIOSubsystem = new DigitalIOSubsystem( this );
	}	// USB_DIO_Family()

	/**
	 * Gets a reference to the digital I/O subsystem of this device.
	 * @return reference to the digital I/O subsystem.
	 */

	public DigitalIOSubsystem dio() {
		return digitalIOSubsystem;
	}	// dio()

	/**
	 * Prints the properties of this device and all of its subsystems. Mainly useful for diagnostic purposes.
	 * @param stream the print stream where properties will be printed.
	 * @return The print stream.
	 */

	public PrintStream print( PrintStream stream ) {
		super.print( stream );
		digitalIOSubsystem.print( stream );
		return stream;
	}	// print()

	// }}}

}	// class USB_DIO_Family

/* end of file */
