/*
 * $RCSfile: Sample.java,v $
 * $Date: 2009/11/30 21:32:09 $
 * $Revision: 1.2 $
 * jEdit:tabSize=4:indentSize=4:collapseFolds=1:
 */

// {{{ build instructions
/*
 * to compile:
 *   javac -cp ../../java/aiousb.jar Sample.java
 * to run:
 *   java -cp ../../java/aiousb.jar:. Sample
 */
// }}}

// {{{ imports
import com.acces.aiousb.*;
import java.io.*;
import java.util.*;
// }}}

class Sample {
	public static final String VERSION = "1.2, 30 November 2009";
	/*
	 * if you want to test writing to the EEPROM, set DEMO_EEPROM_WRITE to 'true'; after writing
	 * to the EEPROM this sample program will attempt to restore the original EEPROM contents,
	 * but you never know ...
	 */
	protected static final boolean DEMO_EEPROM_WRITE = false;

	public static void main( String args[] ) {
/*API*/	USBDeviceManager deviceManager = new USBDeviceManager();
		System.out.println(
			"USB-IIRO-16 sample program version: " + VERSION + "\n"
/*API*/		+ "  AIOUSB Java library version: " + deviceManager.VERSION_NUMBER + ", " + deviceManager.VERSION_DATE + "\n"
/*API*/		+ "  AIOUSB library version: " + deviceManager.getAIOUSBVersion() + ", " + deviceManager.getAIOUSBVersionDate() + "\n"
			+ "  JRE version: " + System.getProperty( "java.version" ) + "\n"
			+ "  OS version: " + System.getProperty( "os.name" )
					+ " " + System.getProperty( "os.arch" )
					+ " " + System.getProperty( "os.version" ) + "\n"
			+ "  This program demonstrates controlling a USB-IIRO-16 device on\n"
			+ "  the USB bus. For simplicity, it uses the first such device found\n"
			+ "  on the bus."
		);
		try {
/*API*/		deviceManager.open().printDevices();
/*API*/		Vector<USBDevice> devices = deviceManager.getDeviceByProductID( deviceManager.USB_IIRO_16 );
			if( devices.size() > 0 ) {
				USB_DIO_Family device = ( USB_DIO_Family ) devices.get( 0 );
/*API*/			device.reset()
/*API*/				.setCommTimeout( 1000 );

				/*
				 * demonstrate writing EEPROM
				 */
				if( DEMO_EEPROM_WRITE ) {
					System.out.print( "Writing to EEPROM ... " );
					final int offset = 0x100;
					String message = "USB-IIRO-16 sample program";
/*API*/				byte[] prevData = device.customEEPROMRead( offset, message.length() );	// preserve current EEPROM contents
/*API*/				device.setCommTimeout( 10000 )	// writing the entire EEPROM (which we're not doing here) can take a long time
/*API*/					.customEEPROMWrite( offset, message.getBytes() );
/*API*/				String readMessage = new String( device.customEEPROMRead( offset, message.length() ) );
/*API*/				device.customEEPROMWrite( offset, prevData )
/*API*/					.setCommTimeout( 1000 );
					System.out.println(
						readMessage.equals( message )
							? "successful"
							: "failed: data read back did not compare" );
				}	// if( DEMO_EEPROM_WRITE )

				/*
				 * demonstrate reading EEPROM
				 */
				System.out.println( "EEPROM contents:\n"
/*API*/				+ Arrays.toString( device.customEEPROMRead( 0, device.CUSTOM_EEPROM_SIZE ) ) );

				/*
				 * demonstrate DIO configuration
				 */
				System.out.print( "Configuring digital I/O ... " );
/*API*/			boolean[] outputs = new boolean[ device.dio().getNumPorts() ];
				Arrays.fill( outputs, true );	// all ports are outputs
/*API*/			boolean[] values = new boolean[ device.dio().getNumChannels() ];
				Arrays.fill( values, false );	// open all relays
/*API*/			device.dio().configure( false, outputs, values );
				System.out.println( "successful" );

				/*
				 * demonstrate writing outputs individually
				 */
				System.out.println(
					  "In the following demonstrations, you should hear the relays click for the first\n"
					+ "16 channels. The latter 16 channels are not connected to relays and will turn\n"
					+ "on and off silently. Also. the LED on the device should blink during these demos."
				);
				System.out.print( "  CLosing relays:" );
/*API*/			for( int channel = 0; channel < device.dio().getNumChannels(); channel++ ) {
					System.out.print( " " + channel );
/*API*/				device.dio().write( channel, true );	// close relay
					Thread.sleep( 100 );
				}	// for( int channel ...
				System.out.print( "\n  Opening relays:" );
/*API*/			for( int channel = 0; channel < device.dio().getNumChannels(); channel++ ) {
					System.out.print( " " + channel );
/*API*/				device.dio().write( channel, false );	// open relay
					Thread.sleep( 100 );
				}	// for( int channel ...
				System.out.println();

				/*
				 * demonstrate writing outputs as a group
				 */
				System.out.print( "Closing all relays ... " );
				Arrays.fill( values, true );	// close all relays
/*API*/			device.dio().write( 0, values );
				System.out.println( "successful" );
				Thread.sleep( 2000 );
				System.out.print( "Opening all relays ... " );
				Arrays.fill( values, false );	// open all relays
/*API*/			device.dio().write( 0, values );
				System.out.println( "successful" );
			}	// if( devices.size() ...
/*API*/		deviceManager.close();
		} catch( Exception ex ) {
			System.err.println( "Error \'" + ex.toString() + "\' occurred while manipulating device" );
/*API*/		if( deviceManager.isOpen() )
/*API*/			deviceManager.close();
		}	// catch( ...
	}	// main()

}	// class Sample

/* end of file */
